from recolul.duration import Duration
from recolul.recoru.attendance_chart import AttendanceChart, ChartRow

_MIN_HOURS_FOR_MANDATORY_BREAK = Duration(6 * 60)


def get_work_time(chart_row: ChartRow) -> Duration:
    """
    Get work time from the column if available,
    else calculate it from clock-in time and current time
    """
    if work_time := chart_row.work_time:
        return Duration.parse(work_time)

    clock_in_time = Duration.parse(chart_row.clock_in_time)
    current_work_time = Duration.now() - clock_in_time
    break_time = (
        Duration(60) if current_work_time >= _MIN_HOURS_FOR_MANDATORY_BREAK
        else Duration(0)
    )
    return current_work_time - break_time


def get_overtime_history(attendance_chart: AttendanceChart) -> tuple[list[str], list[Duration]]:
    days = []
    overtime_history = []
    for row in attendance_chart:
        if not row.clock_in_time:
            continue

        day = row.day
        days.append(day.text)
        required_time = Duration(0 if day.color in ["blue", "red"] else 8 * 60)  # No required hours for holidays
        work_time = get_work_time(row)
        overtime_history.append(work_time - required_time)

    return days, overtime_history


def get_overtime_balance(attendance_chart: AttendanceChart) -> Duration:
    _, history = get_overtime_history(attendance_chart)
    return sum(history, Duration())


def get_leave_time(attendance_chart: AttendanceChart) -> tuple[Duration, bool]:
    day_base_hours = Duration(8 * 60)
    overtime_balance = get_overtime_balance(attendance_chart[:-1])
    last_clock_in = Duration.parse(attendance_chart[-1].clock_in_time)

    leave_time = last_clock_in + day_base_hours - overtime_balance

    # When more than 6 hours must be achieved during the day,
    # add a mandatory 1-hour break time.
    required_today = day_base_hours - overtime_balance
    if required_today > _MIN_HOURS_FOR_MANDATORY_BREAK:
        leave_time += Duration(60)
        return leave_time, True

    return leave_time, False



from enum import Enum
from typing import TypeAlias

from bs4 import Tag


class ChartColumn(str, Enum):
    DATE = "日付"
    WORKPLACE = "作業場所"
    CATEGORY = "勤務区分"
    START = "開始"
    END = "終了"
    WORK_TIME = "労働時間"


class ChartEntry:
    """Single cell of the attendance chart"""
    def __init__(self, tag: Tag):
        self._tag = tag

    @property
    def text(self) -> str:
        return self._tag.text.strip()

    @property
    def color(self) -> str:
        label = self._tag.find("label", recursive=False)
        if not label:
            return ""
        return label.attrs\
            .get("style", "")\
            .removeprefix("color: ")\
            .removesuffix(";")


class ChartHeader:
    """Header of the attendance chart"""
    def __init__(self, tag: Tag):
        self._column_indices: dict[ChartColumn, int] = {
            ChartEntry(column).text.strip(): i
            for i, column in enumerate(tag.find_all("td", recursive=False))
        }

    def get_column_index(self, column: ChartColumn) -> int:
        return self._column_indices[column]

    def has_column(self, column: ChartColumn) -> bool:
        return column in self._column_indices


class ChartRow:
    """Row of the attendance chart"""
    def __init__(self, header: ChartHeader, tag: Tag):
        self._header = header
        self._tag = tag

    def __getitem__(self, column: ChartColumn) -> ChartEntry:
        column_index = self._header.get_column_index(column)
        column = self._tag.select_one(f"td:nth-child({column_index + 1})", recursive=False)
        return ChartEntry(column)

    @property
    def day(self) -> ChartEntry:
        return self[ChartColumn.DATE]

    @property
    def clock_in_time(self) -> str:
        return self[ChartColumn.START].text

    @property
    def clock_out_time(self) -> str:
        return self[ChartColumn.END].text

    @property
    def work_time(self) -> str:
        return self[ChartColumn.WORK_TIME].text


AttendanceChart: TypeAlias = list[ChartRow]
